@@ -32,6 +32,8 @@ using std::size_t;
       CALL(SUBMATCOVACC(double),         SUBMATCOVACC_VAL(double))             \
     case 4:                                                                    \
       CALL(SUBMATCOVACC(int),            SUBMATCOVACC_VAL(int))                \
+    case 6:                                                                    \
+      CALL(SUBMATCOVACC(float),          SUBMATCOVACC_VAL(float))              \
     case 1:                                                                    \
       CALL(SUBMATCOVACC(unsigned char),  SUBMATCOVACC_VAL(unsigned char))      \
     case 2:                                                                    \
@@ -56,6 +58,8 @@ using std::size_t;
       CALL(SUBMATCOVACC(double))                                               \
     case 4:                                                                    \
       CALL(SUBMATCOVACC(int))                                                  \
+    case 6:                                                                    \
+      CALL(SUBMATCOVACC(float))                                                \
     case 1:                                                                    \
       CALL(SUBMATCOVACC(unsigned char))                                        \
     case 2:                                                                    \
@@ -72,7 +76,7 @@ using std::size_t;
 template<typename T>
 class SubMatCovAcc : public SubBMAcc<T> {
 public:
-  SubMatCovAcc(const FBM * xpBM,
+  SubMatCovAcc(FBM * xpBM,
                const IntegerVector& row_ind,
                const IntegerVector& col_ind,
                const NumericMatrix& covar)
@@ -114,7 +118,7 @@ protected:
 
 class RawSubMatCovAcc : public SubMatCovAcc<unsigned char> {
 public:
-  RawSubMatCovAcc(const FBM * xpBM,
+  RawSubMatCovAcc(FBM * xpBM,
                   const IntegerVector& row_ind,
                   const IntegerVector& col_ind,
                   const NumericMatrix& covar,
